/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: MsgDrvCreator_main.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#include <iostream>
#if defined(WIN32)
#include <windows.h>
#include <io.h>
#else
#include <unistd.h>
#endif

#include <fstream>
#include <string>

#include "ServerCommon.h"
#include "MsgDrvCreator.h"
#if !defined(NO_LOGGER)
#include "Logger.h"
#endif

#include "PIDFactory.h"
#include "SrvConfAttrHandler.h"

using namespace std;
using namespace OpenSOAP;

int
main(int argc, char** argv) {

#if defined(NO_LOGGER)
  string logFile;
  SrvConfAttrHandler* srvConf = new SrvConfAttrHandler();
  vector<string> values;
  string query = SERVER_CONF_TAG + "/log/path=?";
  if (0 > srvConf->queryXml(query, values)) {
    logFile = OPENSOAP_TMP_PATH + "log/msgDrvCreator";
  }
  else {
    logFile = values[0] + "msgDrvCreator.log";
  }
  delete srvConf;
  
  FILE* outfp = fopen(logFile.c_str(), "w");
  if (!outfp) {
    return 1;
  }
  if (argc <=1) {
    fprintf(outfp, "invalid initial argument. argc=(%d)\n", argc);
    fprintf(outfp, "exit.\n");
    fclose(outfp);
    return EXIT_FAILURE;
  }
  
  int old;
  int olderr;
  //backup stdout
  old = _dup(1);
  //stdout -> outfp
  if (-1 == _dup2(_fileno(outfp), 1)) {
    fprintf(outfp, "_dup2 to stdout failed.\n");
    fprintf(outfp, "exit.\n");
    fclose(outfp);
    return EXIT_FAILURE;
  }
  //backup stderr
  olderr = _dup(2);
  //stderr -> outfp
  if (-1 == _dup2(_fileno(outfp), 2)) {
    fprintf(outfp, "_dup2 to stderr failed.\n");
    fprintf(outfp, "exit.\n");
    fclose(outfp);
    return EXIT_FAILURE;
  }
#else
  // set log level
  Logger::InitOpt opt;
  opt.outputLevel = LOG_DEBUG;
  Logger::init(opt);
  //write start message
  Logger::printLog(NULL, " MsgDrvCreator_main start", LOG_INFO,
		   0x00100003L, "exe");
#endif

#if !defined(WIN32)
  // make it to daemon & write pid file

  daemon(0, 1);


  PIDFactory* pidFactory = new PIDFactory();
  pidFactory->makePID(MSGDRV_PID_FILE);
  delete pidFactory;

#endif

  // create instance of Message Driver and run  
  ChannelManager* chnlMngr = new MsgDrvCreator();
  chnlMngr->setMask(0x000);
  //chnlMngr->setMask(0x077);
  chnlMngr->run();
  
  delete chnlMngr;
  
#if defined(NO_LOGGER)
	fclose(outfp);
	_dup2(old, 1);
	_dup2(olderr, 2);
#else
  // write stop message
  Logger::printLog(NULL, " MsgDrvCreator_main stop", LOG_INFO,
		   0x00100003L, "exe");
  Logger::term();
#endif

  return EXIT_SUCCESS;
}
