/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: QueueManager_main.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#include <iostream>
#if defined(WIN32)
#include <windows.h>
#include <io.h>
#else
#include <unistd.h>
#endif

#include <string>
#include <fstream>

#if !defined(NO_LOGGER)
#include "Logger.h"
#endif
#include "ServerCommon.h"

#include "QueueManager.h"
#include "PIDFactory.h"
#include "SrvConfAttrHandler.h"

using namespace std;
using namespace OpenSOAP;

int 
main(int argc, char* argv[])
{
  // if argument exists, it behaves as Queue Manager of Forwarder
  bool isFwdQueue = false;

#if defined(WIN32)
  if (argc > 2 && (atoi(argv[2]) != 0)) {
    isFwdQueue = true;
  }
#else
  if (argc > 1 && (atoi(argv[1]) != 0)) {
    isFwdQueue = true;
  }
#endif

#if defined(NO_LOGGER)
  string logFile;
  SrvConfAttrHandler* srvConf = new SrvConfAttrHandler();
  vector<string> values;
  string query = SERVER_CONF_TAG + "/log/path=?";
  if (0 > srvConf->queryXml(query, values)) {
    if (isFwdQueue) {
      logFile = OPENSOAP_TMP_PATH + "log/queueManager_fwd";
    }
    else {
      logFile = OPENSOAP_TMP_PATH + "log/queueManager_req";
    }
  }
  else {
    if (isFwdQueue) {
      logFile = values[0] + "queueManager_fwd.log";
    }
    else {
      logFile = values[0] + "queueManager_req.log";
    }
  }
  delete srvConf;

  FILE* outfp = fopen(logFile.c_str(), "w");
  if (!outfp) {
    return 1;
  }
  if (argc <=1) {
    fprintf(outfp, "invalid initial argument. argc=(%d)\n", argc);
    fprintf(outfp, "exit.\n");
    fclose(outfp);
    return EXIT_FAILURE;
  }
  int old;
  int olderr;
  //backup stdout
  old = _dup(1);
  //stdout -> outfp
  if (-1 == _dup2(_fileno(outfp), 1)) {
    fprintf(outfp, "_dup2 to stdout failed.\n");
    fprintf(outfp, "exit.\n");
    fclose(outfp);
    return EXIT_FAILURE;
  }
  //backup stderr
  olderr = _dup(2);
  //stderr -> outfp
  if (-1 == _dup2(_fileno(outfp), 2)) {
    fprintf(outfp, "_dup2 to stderr failed.\n");
    fprintf(outfp, "exit.\n");
    fclose(outfp);
    return EXIT_FAILURE;
  }
#else
  // set log level  
  Logger::InitOpt opt;
  opt.outputLevel = LOG_DEBUG;
  Logger::init(opt);
  //write start message
  if (isFwdQueue) {
    Logger::printLog(NULL, " QueueManger_main (fwd) start", LOG_INFO,
		     0x00100003L, "exe");
  }
  else {
    Logger::printLog(NULL, " QueueManger_main (req) start", LOG_INFO,
		     0x00100003L, "exe");
  }
#endif

#if !defined(WIN32)
  // make it to daemon & write pid file

  daemon(0, 1);


  PIDFactory* pidFactory = new PIDFactory();
  pidFactory->makePID(isFwdQueue ? FWDQUEUE_PID_FILE : REQQUEUE_PID_FILE);
  delete pidFactory;
  
#endif

  // create instance of QueueManager and run
  ChannelManager* queueManagerPtr = new QueueManager(isFwdQueue);
  queueManagerPtr->run();
  delete queueManagerPtr;

#if defined(NO_LOGGER)
	fclose(outfp);
	_dup2(old, 1);
	_dup2(olderr, 2);
#else
  //write stop message
  if (isFwdQueue) {
    Logger::printLog(NULL, " QueueManger_main (fwd) stop", LOG_INFO,
		     0x00100003L, "exe");
  }
  else {
    Logger::printLog(NULL, " QueueManger_main (req) stop", LOG_INFO,
		     0x00100003L, "exe");
  }
  Logger::term();
#endif

  return 0;
}

